#include <Adafruit_NeoPixel.h>
#include <CapacitiveSensor.h>

#ifdef __AVR__
#include <avr/power.h>
#endif

#define DEBUG true

#define LED_PIN 10

#define NBPIXELS 61
#define STEP 9
#define SAMPLES 30

#define MIN_TRIGGER 2000
#define MAX_TRIGGER 5000

int counter = 0;

const int MORE_LED = 6;
const int LESS_LED = 8;
const int MOOODS_LED = 12;

Adafruit_NeoPixel strip = Adafruit_NeoPixel(NBPIXELS, LED_PIN, NEO_GRB + NEO_KHZ800);

CapacitiveSensor   cs_mooods = CapacitiveSensor(A3, A2);
CapacitiveSensor   cs_less = CapacitiveSensor(A3, A0);
CapacitiveSensor   cs_more = CapacitiveSensor(A3, A1);

String inputString = "";         // a String to hold incoming data
boolean stringComplete = false;  // whether the string is complete

								 // for encoder
/*int val;
int _encoder0PinA = A2;
int _encoder0PinB = A3;
int encoder0PinALast = LOW;
int _n = LOW;*/

int currentHappinessValue = -STEP;

int lasHappinessValue = 0;

uint16_t i = 0;

int brightness = 2;

unsigned long csSum;
int mode = 0; // 1 is level, 2 is for posting

boolean firstrun = true;

void setup() {
	pinMode(LED_BUILTIN, OUTPUT);

	// serial init
	Serial.begin(115200);
	// reserve 200 bytes for the inputString:

	//BLE Init
	//delay(500);
	/*Serial.println("$$$");
	Serial.println("SS,C0");
	Serial.println("R,1");*/
	delay(500); // wait for reboot
	inputString.reserve(200);

	// neo init
	strip.begin();

	for (int i = 0; i < NBPIXELS; i++)
	{
		strip.setPixelColor(i, 0, 0, 0);
	}

	strip.show();

	// rotary encoder init
	//pinMode(encoder0PinA, INPUT);
	//pinMode(encoder0PinB, INPUT);  

	cs_mooods.set_CS_AutocaL_Millis(0xFFFFFFFF);
	cs_more.set_CS_AutocaL_Millis(0xFFFFFFFF);
	cs_less.set_CS_AutocaL_Millis(0xFFFFFFFF);
}

uint32_t getColor(int level) {
	if (level < 10)
		return strip.Color(50, 0, 0);
	if (level < 20)
		return strip.Color(255, 87, 51);
	if (level < 30)
		return strip.Color(224, 51, 255);
	if (level < 40)
		return strip.Color(51, 57, 255);
	if (level < 50)
		return strip.Color(51, 233, 255);
	if (level <= NBPIXELS)
		return strip.Color(63, 255, 51);

	return strip.Color(0, 0, 0);
}

void ReadCapacitiveSensors() {
	Serial.print("\tv: ");
	Serial.print(currentHappinessValue);
	Serial.print("\tm: ");
	Serial.print(mode);
	Serial.print("\tb: ");
	Serial.print(brightness);

#if DEBUG

#endif

	strip.setPixelColor(MORE_LED, strip.Color(0, 0, 0));
	strip.setPixelColor(LESS_LED, strip.Color(0, 0, 0));
	strip.setPixelColor(MOOODS_LED, strip.Color(0, 0, 0));

	long cs = cs_mooods.capacitiveSensor(SAMPLES);
#if DEBUG
	Serial.print("\tc1: ");
	Serial.print(cs);
#endif        
	if (cs > MIN_TRIGGER) {
		csSum += cs;

		if (csSum >= MAX_TRIGGER)
		{
			Serial.print("#");
			if (mode == 3)
			{
				mode = 0;
				//return;
			}
			else {
				if (currentHappinessValue <= 0)
				{
					mode = 3;
					//return;
				}
				else {
					Serial.print("*");
					Serial.println(currentHappinessValue);
					mode = 2;
					if (csSum > 0) { csSum = 0; } //Reset
					cs_mooods.reset_CS_AutoCal(); //Stops readings  
					strip.setPixelColor(MOOODS_LED, strip.Color(0, 0, 255));
					currentHappinessValue = -STEP;
				}
			}
		}
	}
	else {
		csSum = 0; //Timeout caused by bad readings
	}

	cs = cs_more.capacitiveSensor(SAMPLES); //a: Sensor resolution is set to 80
#if DEBUG
	Serial.print("\tc2: ");
	Serial.print(cs);
#endif

	if (cs > MIN_TRIGGER) {
		csSum += cs;

		if (csSum >= MAX_TRIGGER)
		{
			Serial.print("#");
			if (mode == 3)
			{
				brightness += 1;
				if (brightness > 4)
					brightness = 4;				
			}
			else {
				mode = 1;
				if (csSum > 0) { csSum = 0; } //Reset
				cs_more.reset_CS_AutoCal(); //Stops readings  
				strip.setPixelColor(MORE_LED, strip.Color(0, 255, 0));
				currentHappinessValue += STEP;
			}
		}
	}
	else {
		csSum = 0; //Timeout caused by bad readings
	}
	cs = cs_less.capacitiveSensor(SAMPLES); //a: Sensor resolution is set to 80
#if DEBUG
	Serial.print("\tc3: ");
	Serial.print(cs);
#endif        
	if (cs > MIN_TRIGGER) {

		csSum += cs;

		if (csSum >= MAX_TRIGGER)
		{
			Serial.print("#");
			if (mode == 3)
			{
				if (brightness > 0)
					brightness -= 1;				
			}
			else {
				if (csSum > 0) { csSum = 0; } //Reset
				cs_less.reset_CS_AutoCal(); //Stops readings

				strip.setPixelColor(LESS_LED, strip.Color(255, 0, 0));
				currentHappinessValue -= STEP;
			}
		}
	}
	else {
		csSum = 0; //Timeout caused by bad readings
	}

#if DEBUG      
	Serial.println("");
#endif
}

void loop() {


	if (firstrun == true) {
		firstrun = false;
		//delay(5000);
		//demo();
	}

	if (mode == 2) { // en posting		
		for (uint16_t i = 0; i < NBPIXELS; i++) {
			strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
		}
		mode = 0;
	}

	lasHappinessValue = currentHappinessValue;

	ReadCapacitiveSensors();

	//strip.show();
	/*//demo();
	//long start = millis();
	long total2 =  cs_mooods.capacitiveSensor(30);
	//    long total1 =  cs_more.capacitiveSensor(30);
	//  long total3 =  cs_6_7.capacitiveSensor(30);

	Serial.print("> ");
	//Serial.print(millis() - start);        // check on performance in milliseconds
	//Serial.print("\t");                    // tab character for debug windown spacing
	//Serial.print(total1);                  // print sensor output 1
	Serial.print("\t");
	Serial.println(total2);                  // print sensor output 2
	//Serial.print("\t");
	//Serial.println(total3);                // print sensor output 3
	*/

	/*n = digitalRead(encoder0PinA);
	if ((encoder0PinALast == LOW) && (n == HIGH)) {
	if (digitalRead(encoder0PinB) == LOW) {
	encoder0Pos-= STEP;
	} else {
	encoder0Pos += STEP;
	}*/

	if (mode == 0) {
		strip.begin();
		for (uint16_t i = 0; i < NBPIXELS; i++) {
			strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
		}
		currentHappinessValue = 0;
	}

	if (mode == 1) {
		if (currentHappinessValue < 0) {
			currentHappinessValue = 0;
		}

		if (currentHappinessValue > NBPIXELS) {
			currentHappinessValue = NBPIXELS;
		}

		for (uint16_t i = 0; i < NBPIXELS; i++) {
			if (i < currentHappinessValue && i % 2 == 0)
				strip.setPixelColor(NBPIXELS - i - 1, getColor(currentHappinessValue));
			else
				strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
		}

		strip.show();

		if (lasHappinessValue != currentHappinessValue)
		{
			Serial.println(currentHappinessValue);
		}
	}

	if (mode == 2) {
		delay(1000);
		rainbowCycle(20);
		for (uint16_t i = 0; i < NBPIXELS; i++) {
			strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
		}
		//strip.show();
	}

	if (mode == 3) {
		switch (brightness) {
		case 0:
			for (i = 0; i < NBPIXELS; i++) {
				strip.setPixelColor(i, strip.Color(0, 0, 0));
			}
			strip.setPixelColor(NBPIXELS - 1, strip.Color(255, 255, 255));
			break;
		case 1:
			for (i = 0; i < NBPIXELS; i++) {
				if (i % 12 == 0)
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(255, 255, 255));
				else
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
			}
			break;
		case 2:
			for (i = 0; i < NBPIXELS; i++) {
				if (i % 8 == 0)
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(255, 255, 255));
				else
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
			}
			break;
		case 4:
			for (i = 0; i < NBPIXELS; i++) {
				if (i % 2 == 0)
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(255, 255, 255));
				else
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
			}
			break;
		default:
			for (i = 0; i < NBPIXELS; i++) {
				if (i % 4 == 0)
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(255, 255, 255));
				else
					strip.setPixelColor(NBPIXELS - i - 1, strip.Color(0, 0, 0));
			}
			break;
		}
	}

	strip.show();

	//encoder0PinALast = n;
	//TODO manage debounce

	if (stringComplete) {
		// clear the string:
		inputString = "";
		stringComplete = false;
	}

	delay(10);
}

/*
SerialEvent occurs whenever a new data comes in the hardware serial RX. This
routine is run between each time loop() runs, so using delay inside loop can
delay response. Multiple bytes of data may be available.
*/
void serialEvent() {
	while (Serial.available()) {
		digitalWrite(LED_BUILTIN, HIGH);

		// get the new byte:
		char inChar = (char)Serial.read();
		// add it to the inputString:
		inputString += inChar;
		//Serial.println(inChar);

		// if the incoming character is a newline, set a flag so the main loop can
		// do something about it:
		if (inChar == 'a') {
			//demo();
		}
		if (inChar == '\n') {
			stringComplete = true;
			//strip.setPixelColor(counter, strip.Color(0, 0, 0));
			//strip.setPixelColor(counter++, strip.Color(0, 150, 0)); // Moderately bright green color.        
			//strip.show();
		}
		digitalWrite(LED_BUILTIN, LOW);
	}
}

// For randown effects ; https://github.com/1ChicagoDave/Random_NeoPixel_Effects

void demo() {

	rainbow(20);
	rainbowCycle(20);

	// Some example procedures showing how to display to the pixels:
	colorWipe(strip.Color(255, 0, 0), 20); // Red
	colorWipe(strip.Color(0, 255, 0), 20); // Green
	colorWipe(strip.Color(0, 0, 255), 20); // Blue
	//colorWipe(strip.Color(0, 0, 0, 255), 50); // White RGBW
	// Send a theater pixel chase in...
	//theaterChase(strip.Color(127, 127, 127), 50); // White
	//theaterChase(strip.Color(127, 0, 0), 50); // Red
	//theaterChase(strip.Color(0, 0, 127), 50); // Blue


	//theaterChaseRainbow(50);
}

// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint8_t wait) {
	for (uint16_t i = 0; i < NBPIXELS; i++) {
		strip.setPixelColor(i, c);
		strip.show();
		delay(wait);
	}
}

void rainbow(uint8_t wait) {
	uint16_t i, j;

	for (j = 0; j < 256; j++) {
		for (i = 0; i < NBPIXELS; i++) {
			strip.setPixelColor(i, Wheel((i + j) & 255));
		}
		strip.show();
		delay(wait);
	}
}

// Slightly different, this makes the rainbow equally distributed throughout
void rainbowCycle(uint8_t wait) {
	uint16_t i, j;

	for (j = 0; j < 256 * 5; j++) { // 5 cycles of all colors on wheel
		for (i = 0; i < NBPIXELS; i++) {
			strip.setPixelColor(i, Wheel(((i * 256 / NBPIXELS) + j) & 255));
		}
		strip.show();
		delay(wait);
	}
}

//Theatre-style crawling lights.
void theaterChase(uint32_t c, uint8_t wait) {
	for (int j = 0; j < 10; j++) {  //do 10 cycles of chasing
		for (int q = 0; q < 3; q++) {
			for (uint16_t i = 0; i < NBPIXELS; i = i + 3) {
				strip.setPixelColor(i + q, c);    //turn every third pixel on
			}
			strip.show();

			delay(wait);

			for (uint16_t i = 0; i < NBPIXELS; i = i + 3) {
				strip.setPixelColor(i + q, 0);        //turn every third pixel off
			}
		}
	}
}

//Theatre-style crawling lights with rainbow effect
void theaterChaseRainbow(uint8_t wait) {
	for (int j = 0; j < 256; j++) {     // cycle all 256 colors in the wheel
		for (int q = 0; q < 3; q++) {
			for (uint16_t i = 0; i < NBPIXELS; i = i + 3) {
				strip.setPixelColor(i + q, Wheel((i + j) % 255));    //turn every third pixel on
			}
			strip.show();

			delay(wait);

			for (uint16_t i = 0; i < NBPIXELS; i = i + 3) {
				strip.setPixelColor(i + q, 0);        //turn every third pixel off
			}
		}
	}
}

// Input a value 0 to 255 to get a color value.
// The colours are a transition r - g - b - back to r.
uint32_t Wheel(byte WheelPos) {

	WheelPos = 255 - WheelPos;
	if (WheelPos < 85) {
		return strip.Color(255 - WheelPos * 3, 0, WheelPos * 3);
	}
	if (WheelPos < 170) {
		WheelPos -= 85;
		return strip.Color(0, WheelPos * 3, 255 - WheelPos * 3);
	}
	WheelPos -= 170;
	return strip.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

void randomPixelColor(int wait)
{
	int i = random(NBPIXELS);

	uint32_t c = random(2147483647);
	strip.setPixelColor(i, c);
	strip.show();
	delay(wait);

}    // end void randomPixelColor()

void progressiveRandomPixelColor(int wait)
{
	for (uint16_t i = 0; i < NBPIXELS; i++)
	{
		uint32_t c = random(2147483647);
		strip.setPixelColor(i, c);
		strip.show();
		delay(wait);
	}    // end for
}    // end void randomPixelColor()